<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\campaignmonitor
 * @copyright Copyright (c) 2013 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * campaignmonitorInstall Class
 *
 * Class for managing the installation of the Campaign Monitor module.
 *
 * @package clearFusionCMS\module\campaignmonitor
 */
class campaignmonitorInstall implements cfInstallInterface {

	/**
	 * The last error message.
	 *
	 * @var string
	 */
	protected $lastError = '';

	/**
	 * Return the last error.
	 *
	 * @return string The last error string.
	 */
	function getLastError() {
		return $this->lastError;
	}

	/**
	 * Install the module.
	 *
	 * @param flDb The database object to use.
	 * @return bool true if installed; else false.
	 */
	function install($db) {
		$siteMgr = new cfSiteManager($db);
		$sites = $siteMgr->listSites();
		foreach($sites as $site => $n) {
			$this->createSite($db, $site);
		}

		// Create permissions
		$ac = new flAcMgr($db);
		$ac->createPermissionGroup('cfCampaignMonitor', __noop('Campaign Monitor'), 'campaignmonitorBackend'); // 4000
		$ac->createPermission('cfCampaignMonitor.settings', __noop('Can edit Campaign Monitor settings.'), 4010, 'campaignmonitorBackend');

		// Capture events
		$em = clearFusionCMS::getEventManager();
		$em->addListener('newsletter.subscribe', 'campaignmonitorEvents', 'onSubscribe');
		$em->addListener('newsletter.unsubscribe', 'campaignmonitorEvents', 'onUnsubscribe');
		$em->addListener('newsletter.isSSubscribed', 'campaignmonitorEvents', 'onIsSubscribed');

		return true;
	}

	/**
	 * Uninstall the module.
	 *
	 * @param flDb The database object to use.
	 * @return bool true if uninstalled; else false.
	 */
	function uninstall($db) {
		// Remove listeners
		$em = clearFusionCMS::getEventManager();
		$em->removeListener('newsletter.subscribe', 'campaignmonitorEvents', 'onSubscribe');
		$em->removeListener('newsletter.unsubscribe', 'campaignmonitorEvents', 'onUnsubscribe');
		$em->removeListener('newsletter.isSubscribed', 'campaignmonitorEvents', 'onIsSubscribed');

		$siteMgr = new cfSiteManager($db);
		$sites = $siteMgr->listSites();
		foreach($sites as $site => $n) {
			$this->deleteSite($db, $site);
		}

		// Remove permissions
		$ac = new flAcMgr($db);
		$ac->deletePermissionGroup('cfCampaignMonitor');

		// Tell everyone this module has been removed
		clearFusionCMS::getEventManager()->notify(new flEvent('system.moduleRemoved', array(
			'module' => 'campaignmonitor'
			)));

		return true;
	}

	/**
	 * Upgrade the module.
	 *
	 * @param flDb The database object to use.
	 * @param string $installedVer The installed version.
	 * @return bool true if upgraded; else false.
	 */
	function upgrade($db, $installedVer) {
		if(version_compare($installedVer, '1.1.0') < 0) {
			$ac = new flAcMgr($db);

			// Set text domain for existing permission groups and permissions
			$ac->updatePermissionGroup('cfCampaignMonitor', __noop('Campaign Monitor'), 'campaignmonitorBackend');
			$ac->updatePermission('cfCampaignMonitor.settings', __noop('Can edit Campaign Monitor settings.'), 'campaignmonitorBackend');
		}

		if(version_compare($installedVer, '1.1.1') < 0)
			clearFusionCMS::getEventManager()->addListener('newsletter.isSubscribed', 'campaignmonitorEvents', 'onIsSubscribed');

		if(version_compare($installedVer, '1.2.1') < 0) {
			$cfg = new flConfigDb('cfCampaignMonitor', $db);
			$siteMgr = new cfSiteManager($db);
			$sites = $siteMgr->listSites();
			foreach($sites as $site => $n) {
				$ncfg = $siteMgr->getSiteCfg('cfCampaignMonitor', $site);
				$ncfg->apiKey = $cfg->apiKey;
				$ncfg->clientId = $cfg->clientId;
			}
			$cfg->deleteNamespace();
		}

		return true;
	}

	/**
	 * Create a new site.
	 *
	 * @param flDb $db The database object.
	 * @param string $site The short name of the site.
	 * @return bool true if the site was created; else false.
	 */
	function createSite($db, $site) {
		// Create the config
		$siteMgr = new cfSiteManager($db);
		$cfg = $siteMgr->getSiteCfg('cfCampaignMonitor', $site);
		$cfg->apiKey = '';
		$cfg->clientId = '';
		return true;
	}

	/**
	 * Delete a site.
	 *
	 * @param flDb $db The database object.
	 * @param string $site The name of the site to delete.
	 * @return bool true if the site was deleted; else false.
	 */
	function deleteSite($db, $site) {
		$siteMgr = new cfSiteManager($db);
		$cfg = $siteMgr->getSiteCfg('cfCampaignMonitor', $site);
		$cfg->deleteNamespace();
		return true;
	}
}
